@@ -142,15 +142,12 @@ def testConsistencyChecks():
     print(x)
 
 def testvQ():
-    nameFileLEDET = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\D2\D2_0.xlsx"
+    nameFileLEDET = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_0.xlsx"
+    RoxieFile = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_All_WithIron_WithSelfField.map2d"
     a = ParametersLEDET()
     a.readLEDETExcel(nameFileLEDET)
-    Sw = MinMaxSweep(a, 1)
-    I00 = 12330
-    Sw.addCurrentSweep(I00, 1)
-    nameMagnet = "D2"
-    Sw.prepareSimulation(nameMagnet, "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\D2",
-                         ROXIE_File="C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\D2\D2_All_WithIron_WithSelfField.map2d")
+    a.adjust_vQ(RoxieFile)
+    return a
 
 def testQuenchPlanAnalysis(FileName_TDMS, FileName_Sim):
     a = QuenchPlanAnalysis(FileName_TDMS, FileName_Sim)
@@ -233,7 +230,9 @@ def testQuenchPlanAnalysis(FileName_TDMS, FileName_Sim):
 # test_SetUpSimulations()
 # test_EvaluateSimulations()
 
-CompleteRun()
+# CompleteRun()
+
+a  = testvQ()
 
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810111846_a008(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810171217_a041(0).tdms"
