@@ -29,6 +29,7 @@ class TDMSdata:
     t_steps_MF:  float = 0.0
     t_d_MF: float = 0.0
     trigger_PC: int = 0
+    trigger_shoot: int = 0
     I_DCCT_MF: np.ndarray = np.array([])
     I_DCCT_HF: np.ndarray = np.array([])
     I_CLIQ: np.ndarray = np.array([])
@@ -43,6 +44,7 @@ class SIMdata:
     FileName: str = ''
     TimeFrame: np.ndarray = np.array([])
     trigger_PC: int = 0
+    XY_mag_ave: np.ndarray = np.array([])
     I_CoilSections: np.ndarray = np.array([])
     I_CLIQ: np.ndarray = np.array([])
     directionsCLIQ: np.ndarray = np.array([])
@@ -462,7 +464,7 @@ class QuenchPlanAnalysis():
                 chName = chName.replace(" ", "")
                 try:
                     res = temp_Sections.match(chName).groups()
-                    ChN = res + (str(res[0]+res[1]+'_'+res[2]+res[3]+"_"+res[4]+res[5]),)+ (channel.name,)
+                    ChN = res + (str(res[0]+res[1]+'_'+res[2]+res[3]+"_"+res[4]+res[5]),) + (str(res[0]+res[1]+'_'+res[4]+res[5]+"_"+res[2]+res[3]),)+ (channel.name,)
                     CSections.append(ChN)
                 except:
                     pass
@@ -484,8 +486,6 @@ class QuenchPlanAnalysis():
         Coils_Diff = sorted(Coils_Diff, key=getKey0)
 
         self.TDMSdata.VoltageVertices = CSections
-        #for index, tuple in enumerate(CSections):
-        #    self.TDMSdata.VoltageVertices = np.append(self.TDMSdata.VoltageVertices,tuple[6])
 
         ##TODO: also construct sth. for Diffs and Coils
 
@@ -521,13 +521,13 @@ class QuenchPlanAnalysis():
         trigger_shoot = np.gradient(trigger_data, self.TDMSdata.t_d_MF)
         trigger_shoot = np.where(abs(trigger_shoot)>self.TDMSdata.Tolerance)[0][0]-int(self.beforePC/self.TDMSdata.t_d_MF)
         self.TDMSdata.trigger_PC = int(self.beforePC/self.TDMSdata.t_d_MF)
+        self.TDMSdata.trigger_shoot = trigger_shoot
         self.TDMSdata.TimeFrame_MF = self.TDMSdata.TimeFrame_MF[trigger_shoot:]-self.TDMSdata.TimeFrame_MF[trigger_shoot]
         self.TDMSdata.I_DCCT_MF = self.TDMSdata.I_DCCT_MF[trigger_shoot:]
         self.TDMSdata.I_CLIQ = self.TDMSdata.I_CLIQ[trigger_shoot:]
         self.TDMSdata.U_CLIQ = self.TDMSdata.U_CLIQ[trigger_shoot:]
         self.__FillHeaterArrays(I_Heaters, trigger_shoot)
         self.__FillHeaterArrays(U_Heaters, trigger_shoot)
-
         self.__CreateVoltageVertices()
         return
 
@@ -555,6 +555,7 @@ class QuenchPlanAnalysis():
             self.SIMdata.I_QH = I_QH[:][trigger_shoot:]
             U_QH = np.array(f.get("U_QH"))
             self.SIMdata.U_QH = U_QH[:][trigger_shoot:]
+            self.SIMdata.XY_mag_ave = np.array(f.get("XY_MAG_ave"))
         else:
             print("Please provide .mat file for Simulation.")
 
@@ -564,20 +565,25 @@ class QuenchPlanAnalysis():
         return QL
 
     def __calculateAllQuenchIntegrals(self):
-        ## QL Integral 1 - full transient
+        ## QL Integral 1 - from Quench
+        ## Find quench
+        ## Cut parts and
+
+        ## QL Integral 2  - full transient
         ## QL Measured Data
-        self.TDMSdata.QL1 = self.__calculateQuenchIntegral(self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_PC:], self.TDMSdata.I_DCCT_MF[self.TDMSdata.trigger_PC:])
+        self.TDMSdata.QL2 = self.__calculateQuenchIntegral(self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_PC:],
+                                                           self.TDMSdata.I_DCCT_MF[self.TDMSdata.trigger_PC:])
         ## QL Sim Data
-        if abs(sum(self.SIMdata.I_CLIQ))>0:
+        if abs(sum(self.SIMdata.I_CLIQ)) > 0:
             idxC = np.where(self.SIMdata.directionsCLIQ > 0)[0][0]
-            self.SIMdata.QL1 = self.__calculateQuenchIntegral(self.SIMdata.TimeFrame[self.SIMdata.trigger_PC:,0], self.SIMdata.I_CoilSections[self.SIMdata.trigger_PC:,idxC])
+            self.SIMdata.QL2 = self.__calculateQuenchIntegral(self.SIMdata.TimeFrame[self.SIMdata.trigger_PC:, 0],
+                                                              self.SIMdata.I_CoilSections[self.SIMdata.trigger_PC:,
+                                                              idxC])
         else:
-            self.SIMdata.QL1 = self.__calculateQuenchIntegral(self.SIMdata.TimeFrame[self.SIMdata.trigger_PC:,0], self.SIMdata.I_CoilSections[self.SIMdata.trigger_PC:,0])
+            self.SIMdata.QL2 = self.__calculateQuenchIntegral(self.SIMdata.TimeFrame[self.SIMdata.trigger_PC:, 0],
+                                                              self.SIMdata.I_CoilSections[self.SIMdata.trigger_PC:, 0])
 
-        ## QL Integral 2
-        ## tbd
-
-    def GetTDMSCoilStructure(self):
+    def _GetTDMSCoilStructure(self):
         CoilCount = 0
         for i in range(len(self.TDMSdata.VoltageVertices)):
             temp = self.TDMSdata.VoltageVertices[i][1]
@@ -588,18 +594,101 @@ class QuenchPlanAnalysis():
                   " to ", self.TDMSdata.VoltageVertices[i][4]+self.TDMSdata.VoltageVertices[i][5])
         return
 
-    def ProvideTurnsToCoilStructure(self, Coil):
-        VerticeCount = 0
-        for i in range(len(Coil)):
-            CS = Coil[i]
-            for j in range(len(CS)):
-                if len(CS[j]) == 0: continue
-                else:
-                    Vertice = self.TDMSdata.VoltageVertices[VerticeCount][6]
-                    self.MergedVoltages[Vertice] = CS[j]
-                    VerticeCount = VerticeCount +1
+    def FindQuenchTDMS(self, Vthreshold, ValidationTime, Plot=1):
+        tSteps = int(ValidationTime/self.TDMSdata.t_d_MF)
+        found = 0
+
+        if not self.MergedVoltages:
+            print("Please first provide Coil-Structure. ")
+            print("Obj.ProvideTurnsToCoilStructure(self, TurnsToVtaps: Dict)")
+            return
+
+        for key in self.MergedVoltages.keys():
+            Udata = self.TDMSdata.groupMF[key].data
+            if self.TDMSdata.trigger_shoot > 10000:
+                Udata = Udata - np.sum(Udata[:10000]) / 10000
+            else:
+                print("Failed to calculate SNR for Voltage channel.")
+                return
+            Qidx = np.where(abs(Udata[:self.TDMSdata.trigger_shoot+self.TDMSdata.trigger_PC]) > Vthreshold)[0]
+            if len(Qidx)> 0:
+                for i in range(len(Qidx)):
+                    QQidx = np.where(Udata[Qidx[i]:Qidx[i]+tSteps]>Vthreshold)[0]
+                    if len(QQidx) >= (0.95*tSteps) and (Qidx[i]+tSteps)<(self.TDMSdata.trigger_shoot+self.TDMSdata.trigger_PC):
+                        tQuench = Qidx[i]-self.TDMSdata.trigger_shoot
+                        if tQuench < 0:
+                            print("Detected Quench in {} at {}".format(key, (Qidx[i] - self.TDMSdata.trigger_shoot)*self.TDMSdata.t_d_MF))
+                        else:
+                            print("Detected Quench in {} at {}".format(key, self.TDMSdata.TimeFrame_MF[Qidx[i] - self.TDMSdata.trigger_shoot]))
+                        found = 1
+                        if Plot:
+                            try:
+                                fig = plt.figure()
+                                ax = fig.add_subplot(111)
+                                ax.grid(True)
+                                ax.set_title("Detected quench in - "+key)
+                                ax.set_ylim([-3*Vthreshold,3*Vthreshold])
+                                ax.plot(self.TDMSdata.TimeFrame_MF[tQuench-tSteps:tQuench+int(3*tSteps)],Udata[Qidx[i]-tSteps:Qidx[i]+int(3*tSteps)],'--')
+                                ax.axvline(self.TDMSdata.TimeFrame_MF[Qidx[i] - self.TDMSdata.trigger_shoot],c='r')
+                                ax.legend(["$U_{Res,"+key+"}$","$t_{Quench}$"])
+                            except:
+                                pass
+                        break
+
+            # fig = plt.figure()
+            # ax = fig.add_subplot(111)
+            # ax.set_title(key)
+            # ax.plot(np.linspace(0, len(Udata), len(Udata)), Udata, '--')
+        if not found:
+            print("No quench detected.")
+
+    def GetCoilStructure(self, Print = 0, Plot = 1):
+        if Print:
+            for key in self.MergedVoltages.keys():
+                print(key, " --> ", self.MergedVoltages[key])
+        if Plot:
+            fig = plt.figure()
+            ax = fig.add_subplot(111)
+            count = 0
+            legend =[]
+            for key in self.MergedVoltages.keys():
+                idxTap = self.MergedVoltages[key]
+                legend.append(key)
+                ax.scatter(self.SIMdata.XY_mag_ave[0,idxTap.astype(int)-1], self.SIMdata.XY_mag_ave[1,idxTap.astype(int)-1], s = 10, c = self.Colors[count])
+                count = count + 1
+            ax.axis('equal')
+            ax.grid('minor')
+            ax.set_xlabel("x [mm]")
+            ax.set_ylabel("y [mm]")
+            ax.set_title("Turns to V-Taps")
+            ax.legend(legend)
         return
 
+    def ProvideTurnsToCoilStructure(self, Coil):
+        if not self.MergedVoltages:
+            for i in range(len(Coil)):
+                self.MergedVoltages = {**self.MergedVoltages, **Coil[i]}
+            MVcopy = self.MergedVoltages.copy()
+            for old_key in MVcopy.keys():
+                if len(self.MergedVoltages[old_key]) == 0:
+                    self.MergedVoltages.pop(old_key)
+                else:
+                    found = 0
+                    for i in range(len(self.TDMSdata.VoltageVertices)):
+                        if old_key == self.TDMSdata.VoltageVertices[i][6] or  old_key == self.TDMSdata.VoltageVertices[i][7]:
+                            new_key = self.TDMSdata.VoltageVertices[i][8]
+                            self.MergedVoltages[new_key] = self.MergedVoltages.pop(old_key)
+                            found = 1
+                            break
+                    if not found:
+                        print("Couldn't find voltage-tap: {} in TDMS.".format(old_key))
+                        self.MergedVoltages.pop(old_key)
+            return
+        else:
+            self.__CreateTDMSDataObject()
+            self.ProvideTurnsToCoilStructure(Coil)
+            return
+
     def QuenchPlanAnalysis(self, Plot = 0):
         # Plot current together
         if Plot:
